##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################


from OFS.Folder import Folder

from Globals import Persistent, DTMLFile
from Acquisition import Explicit
from OFS.SimpleItem import SimpleItem
from RewriterItem import RewriterItem
from RewriterFolder import RewriterFolder

from BTrees.OOBTree import OOBTree,OOSet

import sys,string,os,exceptions,time


class Rewriter(Folder,Persistent,  SimpleItem):
    """ Rewriter """

    meta_type="Rewriter"

    manage_options= (
        {'label': 'Contents',     
         'action': 'manage_main'},
        {'label': 'Settings',     
         'action': 'manage_settings'},
        {'label': 'Statistics',     
         'action': 'manage_statistics'},
    )
    
    meta_types = (
        { "name":"RewriterItem",
          "action":"manage_addProduct/Rewriter/addRewriterItem"
        },
        { "name":"RewriterFolder",
          "action":"manage_addProduct/Rewriter/addRewriterFolder"
        }
    )


    def __init__(self,id,baseurl):
        """ Constructor """

        self.id      = id
        self.baseurl = baseurl

#        self.requests = OOBTree()
        self.requests = {}



    def setRule(self,old,new,isactive=1):
        """set new Rewriter object"""

        c_obj = self
        comps = old.split('/')

        for comp in comps:
            if not comp: continue

            if comp != comps[-1]:
                if hasattr(c_obj,comp):
                    c_obj = getattr(c_obj,comp)

                else:
                    f = RewriterFolder(comp,comp,'',0)
                    c_obj._setObject(comp,f)
                    c_obj = f

            else:
                if hasattr(c_obj,comp):
                    pass
                else:
                    r = RewriterItem(comp,comp,new,isactive)
                    c_obj._setObject(comp,r)
                    c_obj = r



    def unsetRule(self,path):
        """unset  Rewriter object"""

        obj = self.lookup(path)
        if not obj: raise "RewriterTraversalError", path

        obj.aq_parent._delObject(obj.id)


    def  setPref(self,old,new,isactive):
        """ set rule preferences """
                
        obj = self.lookup(path)
        if not obj: raise "RewriterTraversalError", path

        obj.manage_setPreferences(new,isactive)
   

        

    def lookup(self,path):
        """lookup path"""

        c_obj = self
        comps = path.split('/')

        for comp in comps:
            if not comp: continue

            if hasattr(c_obj,comp):
                c_obj = getattr(c_obj,comp)

            else: return None
   
        return c_obj


    def __call__(self,REQUEST=None,RESPONSE=None):
        """ callable """

        path = REQUEST["PATH_INFO"]

        obj = self.lookup(path)

        if obj:

            if obj.isActive=="on":
                print "redirect on to",obj.newPath
                self._updateRequests(REQUEST)
                self._redirect(RESPONSE,obj.newPath,obj.httpCode)
            else:
                print "redirect is off"
                self._updateRequests(REQUEST,failed=1)
                RESPONSE.redirect(REQUEST["BASE0"] + "/standard_error_message1", lock=1)
 
        else:
            print "not found"
            RESPONSE.setStatus(404)
            RESPONSE.redirect(REQUEST["BASE0"] + "/standard_error_message1", lock=1)


    def _updateRequests(self,REQUEST,failed=0):
        """ update internal requests statistics """

        tp   = (time.time(),REQUEST["REMOTE_ADDR"],REQUEST["HTTP_REFERER"],)

        path = REQUEST["PATH_INFO"]
        if failed==1: path = 'Failed: ' + path

        if  self.requests.has_key(path)==0:
            self.requests[path] = []

        self.requests[path].append(tp)

#        print self.requests



    def _redirect(self,RESPONSE,newPath,httpCode):
        """ """

        if RESPONSE:
            RESPONSE.setStatus(httpCode)
            if newPath.startswith("http://") or newPath.startswith("ftp://"):
                RESPONSE.redirect( newPath ,lock=1 )
            else:
                RESPONSE.redirect( self.baseurl + newPath  ,lock=1) 

        else:
            print "RESPONSE is None" 


    def manage_setPreferences(self,baseurl='',REQUEST=None,RESPONSE=None):
        """save settings """

        self.baseurl = baseurl
        RESPONSE.redirect('manage_main')

            
    
    manage_main       = DTMLFile('dtml/manageFolderworkspace', globals())
    manage_settings   = DTMLFile('dtml/manageRewriter', globals())
    manage_statistics = DTMLFile('dtml/manageStatistics', globals())


manage_addRewriterForm = DTMLFile('dtml/addRewriter', globals())


def manage_addRewriter(self, id,baseurl, title="", REQUEST=None):
    """Add a Rewriter object """

    id      = str(id)
    title   = str(title)
                
    c       = Rewriter(id,baseurl)
    self._setObject(id, c)

    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect(self.absolute_url()+'/manage_main')

